using System;
using System.Web;
using Pegasus.Log4Net;

namespace Pegasus.Web
{
    /// <summary>
    /// Module which strips the sub-domain off all out bound cookies.
    /// I.E.  www.google.com --> google.com
    /// </summary>
    public class CookieModule : IHttpModule
    {
        private static readonly ILog s_logger = LogManager.GetLogger( typeof( CookieModule ) );
        private const char DOT = '.';

        /// <summary>
        /// Generated by .NET
        /// </summary>
        public void Dispose() {}

        /// <summary>
        /// Setup the event for EndRequest
        /// </summary>
        /// <param name="context"></param>
        public void Init( HttpApplication context )
        {
            context.EndRequest += onEndRequest;
        }

        private void onEndRequest( object source, EventArgs e )
        {
            try
            {
                HttpContext context = ( (HttpApplication) source ).Context;

                // retrieve domain and trim off sub-domain
                string host = context.Request.Url.Host;
                int firstDotIdx = host.IndexOf( DOT );
                int lastDotIdx = host.LastIndexOf( DOT );

                if( firstDotIdx == lastDotIdx )
                {
                    // Not enough parts in the host to change it.
                    return;
                }

                // remove the first section.
                string domain = host.Substring( firstDotIdx );

                // replace the domain with the shortened version in all cookies.
                foreach( string name in context.Response.Cookies.AllKeys )
                {
                    context.Response.Cookies[ name ].Domain = domain;
                }
            }
            catch ( Exception ex )
            {
                s_logger.Error( null, ex );
            }
        }
    }
}
